package cs;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;

public class FAHCreditExtracter extends DefaultCreditExtracter {

	private DateFormat dateFormat = new SimpleDateFormat("MM.dd.yy");

	@Override
	public void extractCredit(final Project project) {
		System.out.printf("提取%s项目的用户积分...\n", project.name);
		this.init(project);
		// extract credit
		this.extract(project);
		// new line
		System.out.println();
		System.out.printf("提取%s项目的团队信息...\n", project.name);
		try {
			this.fetchTeamInfo(project);
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
		// new line
		System.out.println();
	}

	private void init(final Project project) {
		this.fetchTeamId(project.teamList);
		this.fetchUserId(project);
	}

	private void fetchTeamId(final List<Team> teamList) {
		Pattern pattern = Pattern.compile("t=(\\d+)");
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("searchchoice", "2");
		for (Team team : teamList) {
			params.put("searchbox", team.name);
			String html = UrlFetchUtil.post("http://folding.extremeoverclocking.com/search.php",
					params);
			if (html == null) {
				throw new RuntimeException("Network error! search.php not found.");
			}
			Document doc = Jsoup.parse(html);
			Elements as = doc.select("td a");
			Element e = null;
			for (Element a : as) {
				String name = a.html()
						.replaceFirst("^(?:\\&nbsp;|\\s*)", "")
						.replaceFirst("(?:\\&nbsp;|\\s*)$", "");
				if (name.equalsIgnoreCase(team.name)) {
					e = a;
					break;
				}
			}
			if (e == null) {
				throw new RuntimeException("team(" + team.name + ") not found.");
			}
			String href = e.attr("href");
			Matcher matcher = pattern.matcher(href);
			if (matcher.find()) {
				team.id = matcher.group(1);
			} else {
				throw new RuntimeException("team(" + team.name + ") id not found.");
			}
		}
	}

	private void fetchUserId(final Project project) {
		Map<String, User> userMap = new HashMap<String, User>();
		for (User user : project.userList) {
			for (String name : user.boincName.split("/")) {
				userMap.put(name.toLowerCase(), user);
			}
		}
		Pattern idPattern = Pattern.compile("u=(\\d+)");
		for (Team team : project.teamList) {
			String url = "http://folding.extremeoverclocking.com/user_list.php?s=&a=1&t=" + team.id;
			boolean isFound = true;
			for (int i = 1; isFound; i++) {
				isFound = false;
				String html = UrlFetchUtil.get(url + "&p=" + i);
				if (html == null) {
					throw new RuntimeException("Network error! user_list not found.");
				}
				Document doc = Jsoup.parse(html);
				Elements as = doc.select("td a");
				for (Element a : as) {
					String href = a.attr("href");
					if (href == null
							|| "".equals(href)
							|| !href.startsWith("/user_summary.php")) {
						continue;
					}
					isFound = true;
					String name = a.html()
							.replaceFirst("^(?:\\&nbsp;|\\s*)", "")
							.replaceFirst("(?:\\&nbsp;|\\s*)$", "");
					User user = userMap.get(name.toLowerCase());
					if (user == null || user.team != team) {
						continue;
					}
					Matcher matcher = idPattern.matcher(href);
					if (matcher.find()) {
						user.id = matcher.group(1);
					} else {
						throw new RuntimeException("user(" + name + ") id not found.");
					}
					// fetch user total credit
					a = a.parent().parent().child(9).child(0);
					user.totalCredit = Double.parseDouble(a.html()
							.replaceFirst("^(?:\\&nbsp;|\\s*)", "")
							.replaceFirst("(?:\\&nbsp;|\\s*)$", "")
							.replaceAll(",", ""));
				}
			}
		}
	}

	private void extract(final Project project) {
		double totalCredit = 0;
		int memberNumber = 0;
		for (User user : project.userList) {
			if (user.id == null) {
				continue;
			}
			String url = "http://folding.extremeoverclocking.com/user_summary.php?s=&u=" + user.id;
			String html = UrlFetchUtil.get(url);
			if (html == null) {
				throw new RuntimeException("Network error! user_summary not found.");
			}
			Document doc = Jsoup.parse(html);
			Elements ths = doc.select("th");
			Element tr = null;
			for (Element th : ths) {
				String title = th.html()
						.replaceFirst("^(?:\\&nbsp;|\\s*)", "")
						.replaceFirst("(?:\\&nbsp;|\\s*)$", "");
				if ("Daily Production".equalsIgnoreCase(title)) {
					tr = th.parent();
					break;
				}
			}
			if (tr == null) {
				throw new RuntimeException("'Daily Production' field not found.");
			}
			tr = tr.nextElementSibling().nextElementSibling();
			boolean isFound = false;
			int interval = project.days;
			for (; tr != null && interval > 0; tr = tr.nextElementSibling()) {
				if (!isFound) {
					String dateString = tr.child(0).html()
							.replaceFirst("^(?:\\&nbsp;|\\s*)", "")
							.replaceFirst("(?:\\&nbsp;|\\s*)$", "");
					Date date = this.parseDate(dateString);
					if (date.getTime() == project.endDate.getTime()) {
						isFound = true;
					}
				}
				if (isFound) {
					String credit = tr.child(1).html()
							.replaceFirst("^(?:\\&nbsp;|\\s*)", "")
							.replaceFirst("(?:\\&nbsp;|\\s*)$", "")
							.replaceAll(",", "");
					user.credit += Double.parseDouble(credit);
					interval--;
				}
			}
			if (interval > 0) {
				throw new RuntimeException("已超出提取时间范围");
			}
			if (user.credit > 0) {
				totalCredit += user.credit;
				memberNumber++;
				user.team.memberNumber++;
			}
			System.out.printf("user: %s, credit: %s\n", user.boincName, user.credit);
		}
		project.memberNumber = memberNumber;
		if (!project.challenge.isTeam) {
			project.totalCredit = totalCredit;
		}
	}

	private void fetchTeamInfo(final Project project) {
		Pattern pattern = Pattern.compile("^([,\\d]+)");
		for (Team team : project.teamList) {
			String url = "http://folding.extremeoverclocking.com/team_summary.php?s=&t=" + team.id;
			String html = UrlFetchUtil.get(url);
			if (html == null) {
				throw new RuntimeException("Network error! team_summary not found.");
			}
			Document doc = Jsoup.parse(html);
			Elements ths = doc.select("th");
			for (Element th : ths) {
				String txt = th.text();
				if ("Rank Overall".equalsIgnoreCase(txt)) {
					th = th.parent().nextElementSibling().child(0);
					team.ranking = Integer.parseInt(th.html().replaceAll(",", ""));
				} else if ("Team User Info".equalsIgnoreCase(txt)) {
					th = th.parent().parent().child(4).child(1);
					txt = th.text();
					Matcher matcher = pattern.matcher(txt);
					if (matcher.find()) {
						team.memberTotalNumber = Integer.parseInt(matcher.group(1).replaceAll(",",
								""));
					} else {
						throw new RuntimeException(
								"Network error! team.memberTotalNumber not found.");
					}
					break;
				}
			}
		}
	}

	private Date parseDate(final String str) {
		Date d = null;
		try {
			d = dateFormat.parse(str);
		} catch (ParseException e) {
			throw new RuntimeException(e);
		}
		return d;
	}
}
