package cs;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

public class RewardCalculator {

	public void evaluate(final Challenge challenge) {
		System.out.println("计算奖分...");
		this.evaluateProjectReward(challenge);
		if (challenge.projectNumber > 1) {
			this.evaluateChallengeReward(challenge);
		}
	}

	private void evaluateProjectReward(final Challenge challenge) {
		// 竞赛规模数
		int scale = 0;
		// ④：国际赛 Team China 团队竞赛规模系数
		float p4 = 1;
		if (challenge.projectNumber > 1) {
			// 多个项目的竞赛
			// 对于同时进行多个项目的竞赛，其中的 ④、⑤、⑥ 取值于全部竞赛项目的平均值
			int teamScale = 0;
			for (Project project : challenge.projectList) {
				scale += project.memberNumber;
				if (challenge.isTeam) {
					teamScale += project.teamNumber;
				}
			}
			scale = scale / challenge.projectNumber;
			if (challenge.isTeam) {
				p4 = this.getTeamScaleRatio(teamScale / challenge.projectNumber);
			}
		}

		for (Project project : challenge.projectList) {
			// 排名
			Collections.sort(project.userList);
			int pos = 0;
			double minCredit = Double.MAX_VALUE;
			for (User user : project.userList) {
				if (user.credit < minCredit) {
					minCredit = user.credit;
					pos++;
				}
				user.ranking = pos;
			}
			// ⑤：国际赛 Team China 团队成员参赛系数
			// ⑥：个人赛规模系数
			float p56 = this.getScaleRatio(scale > 0 ? scale : project.memberNumber);

			// d：竞赛持续天数
			int d = project.days;
			// ①：所有 Team China 竞赛参与者基本奖励积分：d x 3
			int p1 = d * 3;

			// ③：国际赛 Team China 团队排位奖励积分
			int p3 = 0;
			if (challenge.isTeam) {
				p3 = this.getTeamRankingReward(project.challengeRanking);
			}

			// Tc：国际赛中团队总积分 或 个人赛中全部参赛者总积分
			double tc = project.totalCredit;

			/*
			 * Y：项目平衡系数：
			 * 先求出各团队的项目平衡系数，然后以各个团队的参赛队员数为权值进行加权平均
			 * 其中各团队的项目平衡系数的计算公式为（设相应的团队i为Ti）：
			 * 若（Ti的国际排名/Ti的总队员数） < 2，则项目平衡系数为：log（（Ti的国际排名/Ti的总队员数）× 25 + 5）
			 * 否则项目平衡系数为：√log（（Ti的国际排名/Ti的总队员数）× 25 + 5）
			 * */
			float Y = 0;
			for (Team team : project.teamList) {
				float n = team.ranking * 1f / team.memberTotalNumber;
				double y = Math.log10(n * 25 + 5);
				if (n >= 2) {
					y = Math.sqrt(y);
				}
				Y += team.memberNumber * y;
			}
			Y = Y / project.memberNumber;

			for (User user : project.userList) {
				if (user.credit <= 0) {
					user.reward = 0;
					continue;
				}
				int p2 = this.getRankingReward(user.ranking);
				// Pc：个人积分
				double pc = user.credit;
				// X：竞赛活跃度系数
				float X = 1;
				// N：该用户当年参加过的竞赛数
				int N = user.challengeNumber + 1;
				X = (float) Math.log10(N * 10f / challenge.challengeNumber + N);

				double reward = p1 + (p2 + p3 + 1000f * pc / tc) * (1 + 0.07f * (d - 1))
						* p4 * p56
						* X * Y;
				user.reward = (int) Math.round(reward);
				// 最终积分修正
				if (challenge.projectNumber == 1) {
					int r = user.reward;
					// 单个项目竞赛的计算结果最大值为 500（假如多人计算出超过 500 的竞赛积分，则均以 500 为最终结果）
					r = r > 500 ? 500 : r;
					// 最终计算结果进位到十位数（假如刚刚好是10的倍数那就到此为止；如果有零头的话，就进位到10位。这样计算出来的结果就是最终的竞赛积分数）
					if (r % 10 != 0) {
						r = (int) Math.floor(r / 10f) * 10 + 10;
					}
					user.reviseReward = r;
				}
			}
		}
	}

	private void evaluateChallengeReward(final Challenge challenge) {
		// n：竞赛项目数
		int n = challenge.projectNumber;
		// d：竞赛持续天数
		int d = challenge.days;

		int init = d * n;
		float prefix = (16f + d * n) / ((15f + d + n) * n);

		Map<String, UserSummary> userSammaryMap = new HashMap<String, UserSummary>();
		for (Project project : challenge.projectList) {
			int reward = 0;
			for (User user : project.userList) {
				UserSummary us = userSammaryMap.get(user.forumName);
				if (us == null) {
					us = new UserSummary(user.forumName, user.boincName);
					us.userList = new ArrayList<User>(challenge.projectNumber);
					userSammaryMap.put(user.forumName, us);
					reward = Math.round(init + prefix * user.reward);
				} else {
					reward = Math.round(us.reward + prefix * user.reward);
				}
				us.reward = reward;
				us.userList.add(user);
			}
		}

		List<UserSummary> userSummaryList = new ArrayList<UserSummary>(userSammaryMap.values());
		// 排名
		Collections.sort(userSummaryList);
		int pos = 0;
		long minReward = Long.MAX_VALUE;
		for (Iterator<UserSummary> iterator = userSummaryList.iterator(); iterator.hasNext();) {
			UserSummary user = iterator.next();
			if (user.reward == init) {
				iterator.remove();
				continue;
			}
			if (user.reward < minReward) {
				minReward = user.reward;
				pos++;
			}
			user.ranking = pos;
			// 最终积分修正
			// 多个项目竞赛的计算结果最大值为 1200（假如多人计算出超过 1200 的竞赛积分，则均以 1200 为最终结果）
			int reward = user.reward;
			reward = reward > 1200 ? 1200 : reward;
			// 最终计算结果进位到十位数（假如刚刚好是10的倍数那就到此为止；如果有零头的话，就进位到10位。这样计算出来的结果就是最终的竞赛积分数）
			if (reward % 10 != 0) {
				reward = (int) Math.floor(reward / 10f) * 10 + 10;
			}
			user.reviseReward = reward;
		}
		challenge.userSummaryList = userSummaryList;
	}

	private float getScaleRatio(final int scale) {
		if (scale <= 0) {
			throw new IllegalArgumentException("scale <= 0");
		} else if (scale > 0 && scale <= 5) {
			return 0.1f;
		} else if (scale >= 6 && scale <= 10) {
			return 0.2f;
		} else if (scale >= 11 && scale <= 20) {
			return 0.4f;
		} else if (scale >= 21 && scale <= 30) {
			return 0.6f;
		} else if (scale >= 31 && scale <= 50) {
			return 0.8f;
		} else if (scale >= 51 && scale <= 80) {
			return 1.0f;
		} else if (scale >= 81 && scale <= 120) {
			return 1.1f;
		} else if (scale >= 121 && scale <= 200) {
			return 1.2f;
		} else {
			return 1.3f;
		}
	}

	private float getTeamScaleRatio(final int scale) {
		if (scale <= 0) {
			throw new IllegalArgumentException("scale <= 0");
		} else if (scale > 0 && scale <= 5) {
			return 0.1f;
		} else if (scale >= 6 && scale <= 10) {
			return 0.2f;
		} else if (scale >= 11 && scale <= 20) {
			return 0.5f;
		} else if (scale >= 21 && scale <= 30) {
			return 0.8f;
		} else if (scale >= 31 && scale <= 50) {
			return 1.0f;
		} else if (scale >= 51 && scale <= 100) {
			return 1.1f;
		} else if (scale >= 101 && scale <= 200) {
			return 1.2f;
		} else {
			return 1.3f;
		}
	}

	private static final int[] RANKING_REWARD_ARRAY = {
			100, 90, 80, 70, 63, 60, 57, 54, 51, 48, 45, 42, 39, 36, 33, 30, 27, 24, 21, 18, 15,
			12, 9, 6, 3
	};
	private static final int[] TEAM_RANKING_REWARD_ARRAY = {
			100, 90, 80, 70, 60, 50, 45, 40, 35, 30
	};

	private int getRankingReward(final int ranking) {
		if (ranking <= 0) {
			throw new IllegalArgumentException("ranking <= 0");
		} else if (ranking >= 26) {
			return 0;
		}
		return RANKING_REWARD_ARRAY[ranking - 1];
	}

	private int getTeamRankingReward(final int ranking) {
		if (ranking <= 0) {
			throw new IllegalArgumentException("ranking <= 0");
		} else if (ranking >= 11 && ranking <= 15) {
			return 25;
		} else if (ranking >= 16 && ranking <= 20) {
			return 20;
		} else if (ranking >= 21 && ranking <= 25) {
			return 15;
		} else if (ranking >= 26 && ranking <= 30) {
			return 10;
		} else if (ranking >= 31 && ranking <= 40) {
			return 5;
		} else if (ranking >= 41 && ranking <= 50) {
			return 3;
		} else if (ranking >= 51) {
			return 0;
		} else {
			return TEAM_RANKING_REWARD_ARRAY[ranking - 1];
		}
	}
}
